/************************************************************************/
/* netinfo_portable.h- Portability Header				*/
/************************************************************************/
/*
 *  (c)2008 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/

#ifndef _NETINFO_PORTABLE_H_
#define _NETINFO_PORTABLE_H_


#ifndef WIN32

#include <stdint.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/select.h>
#ifdef HAVE_EPOLL
#include <sys/epoll.h>
#endif
#include <unistd.h>
#include <dlfcn.h>
#include <netinet/in.h>
#include <netdb.h>
#include <pthread.h>
#define	DIR_DELIMITER		"/"
#define NETINFO_API


#else //WIN32

#include <stdint.h>
#include <time.h>
#include <winsock2.h>
#include <windows.h>
#include <io.h>
#define	DIR_DELIMITER		"\\"
#ifndef EOVERFLOW
#define EOVERFLOW               75
#endif
#ifndef ENODATA
#define ENODATA                 61
#endif
#ifndef ETIMEDOUT
#define ETIMEDOUT		110
#endif
#define RTLD_NOW                1
#define dlopen(x,y)             LoadLibrary (x)
#define dlsym(x,y)              GetProcAddress (x, y)
#define dlclose(x)              FreeLibrary (x)
#define	ftello(x)		ftell(x)
#define SOCKET_CLEANUP()        WSACleanup()
#define sleep(x)		Sleep (1000 * x)
#define O_NONBLOCK		00004000	/* defined in fcntl.h */
#define F_GETFL			3		/* defined in fcntl.h */
#define F_SETFL			4		/* defined in fcntl.h */

#ifdef BUILD_NETINFO
#define NETINFO_API __declspec(dllexport)
#else
#define NETINFO_API __declspec(dllimport)
#endif


#define	PTHREAD_CREATE_DETACHED		1
typedef	unsigned char			u_int8_t;
typedef	WORD				u_int16_t;
typedef	DWORD				u_int32_t;
typedef	unsigned long long		u_int64_t;
typedef	struct _MutexHolder		pthread_mutex_t;
typedef	struct _HandleHolder		pthread_attr_t;
typedef	struct _HandleHolder		pthread_mutexattr_t;
typedef	struct _HandleHolder		pthread_condattr_t;
typedef	void *				pthread_t;
typedef	struct _HandleHolder		pthread_cond_t;

/* Generic structure for the "portable" pthread_* functions */
struct  _HandleHolder {
        HANDLE          h;
};


/* Mutex structure for the "portable" pthread_* functions */
struct  _MutexHolder {
	CRITICAL_SECTION		h;
};


/* POSIX.1b structure for a time value.  This is like a `struct timeval' but
   has nanoseconds instead of microseconds.  */
/* mingw includes this now
struct timespec
{
    time_t tv_sec;            // Seconds.
    long int tv_nsec;         // Nanoseconds.
};
*/

#define	pthread_mutex_create()  CreateMutex (NULL, FALSE, NULL)
//#define	pthread_mutex_lock(x)   WaitForSingleObject(x, INFINITE)
//#define	pthread_mutex_unlock(x) ReleaseMutex(x)
#define	pthread_exit(x)         ExitThread((DWORD)x)


int pthread_attr_init (pthread_attr_t *attr);
int pthread_attr_setdetachstate (pthread_attr_t *attr, int detachstate);

int pthread_mutex_init (pthread_mutex_t *mutex,
                        pthread_mutexattr_t *mutexattr);
int pthread_create(pthread_t  *thread,
                        pthread_attr_t *attr,
                        void * (*start_routine)(void *),
                        void *arg);

int pthread_cond_init(pthread_cond_t *cond,
                        pthread_condattr_t *cond_attr);
int pthread_cond_destroy(pthread_cond_t *cond);
int pthread_cond_signal (pthread_cond_t *cond);
int pthread_cond_timedwait(pthread_cond_t *cond,
			pthread_mutex_t *mutex,
			const struct timespec *abstime);

#ifndef localtime_r
struct tm *localtime_r (const time_t *time, struct tm *result);
#endif

#endif	//WIN32



int p_gethostname(char *buffer, int len);


#endif	//_NETINFO_PORTABLE_H_

